#!/usr/local/bin/perl

=head1 スクリプト名

wemo.cgi - wemo 言語パーサー & メモ出力

=head1 概要

wemo_save.cgi が生成するwemo 言語で書いたメモテキストを HTML に変換し、CGI 出力を行う。

=cut
use CGI qw(:cgi);
use Time::Local;

# 環境変数の取得
$docroot = "/homepage";#$ENV{'DOCROOT'};
$cgidir = "/cgi-bin";#$ENV{'CGIDIR'};

# メモを格納するディレクトリ名
$memodir = "memo";

# メモのカテゴリのASCII文字列をキーに日本語文字列を連想配列の値として取得
# メモ格納ディレクトリにある category.txt から読み込む
if(open(IN, "<$docroot/$memodir/category_utf-8.txt")){
	while(<IN>){
		chomp;
		($catstr, $category) = split(/\t/);
		$categories{$catstr} = $category;
	}
	close(IN);
}else{
	# category.txt が無ければ、次の連想配列のデータを使う
	%categories = (
		"favor" => "お気に入り",
		"diary" => "日記",
		"memo" => "メモ",
		"sched" => "スケジュール",
		"annot" => "註釈",
		"file" => "マイドキュメント",
	);
}

# メモファイル名の取得
$infile = $ENV{'PATH_INFO'};

# メモファイル名からメモの日付を取得し、日付から曜日を timelocal 関数から取得
($date = $infile) =~ s/^\/memo_(\d{4}-\d{2}-\d{2})\.txt$/$1/i;

# メモの最初の部分の CGI 出力
print <<HEADER;
Content-type: text/html; charset=UTF-8

<HTML>
<HEAD>
<TITLE>Wemo</TITLE>
<META HTTP-EQUIV="Content-Type" content="text/html; charset=UTF-8">
<link rel="stylesheet" type="text/css" href="/mystyle.css">
</HEAD>
<BODY>
<H3>Wemo</H3>
<DL>
HEADER

#%articles = ();
# メモの入力ファイルをオープン
open(IN, "$docroot/$memodir$infile");

while(<IN>){
	chomp;
	$count++;
	# 日付行の次から処理を開始する
	if($_ =~ /^(\d{4}-\d{2}-\d{2} \(\w{3}\.\))$/i){
		$insw = 1;# フラグを立てる
		print "<DT>$1<DD>\n";
		next;
	}
	if($insw == 1){
		if($_ =~ /<.+>/ && $pre != 1){
			next;
		}
		# カテゴリデータの処理
		if($_ =~ /^C:\s+(.*)$/i){
			$category = $1;# カテゴリデータを取得する
			next unless $category;# カテゴリデータがない場合はスキップ
			# カテゴリとタイトルデータが揃ったら処理をする
			# タイトルデータが空の場合はスキップ
			if($title ne ""){
				$catid = time + $catcount;
				$articles{"${category}_$catid"} = $title;
				# CGI 出力
				print "<div class=\"emph\"><A HREF=\"/$memodir/memo_index.html#$category\">[$categories{$category}]</A> <A NAME=\"${category}_$catid\">$title</A></div>\n";
				$title = "";
				$category = "";
				$catcount++;
			}
		# タイトルデータの処理
		}elsif($_ =~ /^T:\s+(.*)$/i){
			$title = $1;# タイトルデータを取得する
			next unless $title;# タイトルデータがない場合はスキップ
			# タイトルにハイパーリンクを張る場合の処理
			$title =~ s/\[U:\s+([^;]+);T:\s+([^\]]+)\]/<a href="$1">$2<\/a>/i;
			$title =~ s/\[U:\s+([^;\]]+)\]/<a href="$1">$1<\/a>/i;
			# カテゴリとタイトルデータが揃ったら処理をする
			# カテゴリデータが空の場合はスキップ
			if($category ne ""){
				$catid = time + $catcount;
				$articles{"${category}_$catid"} = $title;
				# CGI 出力
				print "<div class=\"emph\"><A HREF=\"/$memodir/memo_index.html#$category\">[$categories{$category}]</A> <A NAME=\"${category}_$catid\">$title</A></div>\n";
				$category = "";
				$title = "";
				$catcount++;
			}
		# パラグラフ（段落）データの処理
		}elsif($_ =~ /^(?:DE:|P:)\s+(.*)$/i){
			$_ = $1;
			next unless $_;# データが空の場合はスキップ
			&linking($_);
			print "<p>$_</p>\n";# CGI 出力
			next;
		# 整形済みテキストデータの処理（1）
		}elsif($_ =~ /^PRE:$/i){
			print "<pre>\n";# CGI 出力
			$pre = 1;# フラグを立てる
			next;
		# 整形済みテキストデータの処理（2）
		}elsif($_ =~ /^\/PRE:/i){
			print "</pre><br>\n";# CGI 出力
			$pre = 0;# フラグを元に戻す
			next;
		# 番号なしリストデータの処理（1）
		}elsif($_ =~ /^UL:$/i){
			print "<ul>\n";# CGI 出力
			$ul = 1;# フラグを立てる
			next;
		# 番号なしリストデータの処理（2）
		}elsif($_ =~ /^\/UL:$/i){
			print "</ul><br>\n";# CGI 出力
			$ul = 0;# フラグを元に戻す
			next;
		# 番号付きリストデータの処理（1）
		}elsif($_ =~ /^OL:$/i){
			print "<ol>\n";# CGI 出力
			$ol = 1;# フラグを立てる
			next;
		# 番号付きリストデータの処理（2）
		}elsif($_ =~ /^\/OL/i){
			print "</ol><br>\n";# CGI 出力
			$ol = 0;# フラグを元に戻す
			next;
		# 階層型リストデータの処理（1）
		}elsif($_ =~ /^ULS:$/i){
			$blevel = 0;
			print "<ul>\n";# CGI 出力
			$uls = 1;# フラグを立てる
			next;
		# 階層型リストデータの処理（2）
		}elsif($_ =~ /^\/ULS:$/i){
			while($blevel > 0){
				print "\t" x $blevel;# CGI 出力
				print "</ul>\n";
				$blevel--;
			}
			print "</ul><br>\n";# CGI 出力
			$uls = 0;# フラグを元に戻す
			next;
		# 定義リストデータの処理（1）
		}elsif($_ =~ /^DL:$/i){
			# <dl>タグの前にタブを入れて、メモ開始の<dl>タグと区別
			print "\t<dl>\n";# CGI 出力
			$dl = 1;# フラグを立てる
			next;
		# 定義リストデータの処理（2）
		}elsif($_ =~ /^\/DL/i){
			# </dl>タグの前にタブを入れて、メモ終了の</dl>タグと区別
			print "\t</dl><br>\n";# CGI 出力
			$dl = 0;# フラグを元に戻す
			next;
		# 表データの処理（1）
		}elsif($_ =~ /^ta:$/i){
			$ta = 1;# フラグを立てる
			$count = 0;
			next;
		# 表データの処理（2）
		}elsif($_ =~ /^\/ta:$/i){
			print "</TABLE><BR>\n";# CGI 出力
			$ta = 0;# フラグを元に戻す
			next;
		}elsif($_ =~ /^bq:$/i && $pre != 1){
			$bq = 1;
			print "<BLOCKQUOTE>\n";
			next;
		}elsif($_ =~ /^\/bq:$/i && $pre != 1){
			$bq = 0;
			print "</BLOCKQUOTE>\n";
			next;
		# 改行データの処理（1）
		}elsif($_ =~ /^br:$/i){
			$br = 1;# フラグを立てる
			next;
		# 改行データの処理（2）
		}elsif($_ =~ /^\/br:$/i){
			$br = 0;# フラグを元に戻す
			print "<br>\n";# CGI 出力
			next;
		# memol 言語で記述した画像データの処理
		}elsif($_ =~ /^\[[^\]]+\]$/ && $pre != 1 && $ul != 1 && $ol != 1 && $uls != 1 && $dl != 1 && $ta != 1 && $br != 1 && $bq != 1){
			&linking($_);
			print $_, "\n";# CGI 出力
		# その他の行の処理
		}elsif($_ !~ /^\w{1,3}:$/ && $line !~ /^\w{1,3}: .+$/ && $pre != 1 && $ul !=1 && $ol != 1 && $uls != 1 && $dl != 1 && $ta != 1 && $br != 1 && $bq != 1){ 
			&linking($_);
			print $_, "\n";# CGI 出力
		}
		# フラグが立っている状態でのデータ処理
		# 整形済みテキストデータの処理（3）
		# <>記号のみ実体参照エンコードする
		if($bq == 1){
			&linking($_);
			print "$_\n";
		}
		if($pre == 1){
			$line = &entities_encode($_);
			print $_, "\n";# CGI 出力
		}
		# 番号なしリストデータの処理（3）
		if($ul == 1){
			&linking($_);
			print "<li>$_\n";# CGI 出力
		}
		# 番号付きリストデータの処理（3）
		if($ol == 1){
			&linking($_);
			print "<li>$_\n";# CGI 出力
		}
		# 階層型リストデータの処理（3）
		if($uls == 1){
			&linking($_);
			if($line =~ /^([ \t]*)[^ \t].*$/){
				$level = length $1;
				if($blevel > $level){
					$diff = $blevel - $level;
					if($diff >= 1){
						for($i = 1;$i<=$diff;$i++){
							$ts = "\t" x ($blevel - $i + 1);
							print "$ts</ul>\n";# CGI 出力
						}
					}
					$_ =~ s/^([ \t]*)([^ \t].*)$/$1<li>$2/;
				}elsif($blevel == $level){
					$_ =~ s/^([ \t]*)([^ \t].*)$/$1<li>$2/;
				}elsif($blevel < $level){
					$diff = $level - $blevel;
					if($diff >= 1){
						for($i = 1;$i<=$diff;$i++){
							$ts = "\t" x ($blevel + $i);
							print "$ts<ul>\n";# CGI 出力
						}
					}
					$_ =~ s/^([ \t]*)([^ \t].*)$/$1<li>$2/;
				}
			}
			$blevel = $level;
			print $_,"\n";# CGI 出力
		}
		# 定義リストデータの処理（3）
		if($dl == 1){
			if($_ =~ /^([^:]+:) (.*)$/){
				$prop = $1;
				$str = &linking($2);
				print "\t\t<dt>$prop\n\t\t<dd>$str\n";# CGI 出力
			}
		}
		# 改行データの処理（3）
		if($br == 1){
			&linking($_);
			$_ =~ s/^(.*)$/$1<br>/;
			print $_,"\n";# CGI 出力
		}
		# 表データの処理（3）
		if($ta == 1){
			$count++;
			if($count == 1){
				$caption = &linking($_);
				# CGI 出力
				print "<TABLE BORDER=7>\n<CAPTION ALIGN=top><B>$caption</CAPTION>\n";
			}elsif($count == 2){
				# 表の項目データを取得して、HTMLを出力します。
				&linking($_);
				@heads = split(/:/, $_);
				print "<TH>";# CGI 出力
				for($i=0; $i<$#heads; $i++){
					print "$heads[$i]</TH><TH>";# CGI 出力
				}
				print "$heads[$#heads]</TH>\n";# CGI 出力
			}else{
				# 規則的なテキストデータから1行ずつ変換して出力します。
				&linking($_);
				@fd = split(/:/, $_);
				print "<TR>\n<TD>";# CGI 出力
				for($i=0; $i<$#fd; $i++){
					print "$fd[$i]</TD><TD>";# CGI 出力
				}
				print "$fd[$#fd]</TD>\n</TR>\n";# CGI 出力
			}
		}
	}
}
close(IN);

# メモ HTML 最終部分の CGI 出力
print "</DL>\n<HR>\n";

# メモ作成・編集用フォームの CGI 出力
print "<FORM action=\"$cgidir/wemo_edit.cgi\">\n";
# <<重要>> INPUTタグのhidden属性で、メモの日付 date を $date として
# wemo_edit.cgi に渡すことによって、どのメモファイルを処理するのかが同定される
print "<INPUT TYPE=\"hidden\" NAME=\"date\" VALUE=\"$date\">\n";
# SELECTタグによるカテゴリの選択メニューを出力する
print "<SELECT name=\"category\" size=1>\n";
print "<OPTION selected></OPTION>\n";

# OPTIONタグで、カテゴリの選択メニューを作成する
foreach $catstr (sort keys(%categories)){
	print "<OPTION value=\"$catstr\">$categories{$catstr}</OPTION>\n";
}
# フォームの完成
print "</SELECT><INPUT TYPE=\"submit\" VALUE=\"Create/Edit\"></FORM>\n";

# 註釈メモ作成のための CGI へのリンク出力
#foreach $article (sort keys(%articles)){
#	$title = &juri_encode($articles{$article});
#	print "<a href=\"$cgidir/wemo_edit.cgi?annot=$date&name=$article&title=$title\">Annotate $articles{$article}</a><br>\n";
#}

print "[ <a href=\"$cgidir/wemo_list.cgi\">List</a> ]\n";
print " | [ <a href=\"$cgidir/wemo_calendar.cgi\">Calendar</a> ]\n";

# CGI 出力の終了
print "</BODY>\n</HTML>\n";

# 文字データ中の memol 言語文字列の処理
sub linking{
	$catid = time + $catcount;# アンカー名を確実に区別して生成する

	# ハイパーリンクの処理
	# [u: URL;T: タイトル]
	$_[0] =~ s/\[U:\s+([^;]+);T:\s+([^\]]+)\]/<a href="$1">$2<\/a>/gi;
	# [u: アンカーベース名;t タイトル]
	$_[0] =~ s/\[N:\s+([^;]+);T:\s+([^\]]+)\]/<a name="${1}_$catid">$2<\/a>/i;
	# [u: URL]
	$_[0] =~ s/\[U:\s+([^;\]]+)\]/<a href="$1">$1<\/a>/gi;

	# 画像データの処理
	# [i: 画像URL]
	$_[0] =~ s/\[I:\s+([^;\]]+)\]/<IMG SRC="$1" ALIGN=LEFT HSPACE=20 ALT="$1">/gi;
	# [i: 画像URL;t: タイトル]
	$_[0] =~ s/^\[I:\s+([^;]+);T:\s+([^\]]+)\]$/<IMG SRC="$1" ALIGN=LEFT HSPACE=20 ALT="$1">$2<BR CLEAR=ALL>/i;
	# [i: 画像URL;u: 画像にリンクするURL;t: タイトル]
	$_[0] =~ s/^\[I:\s+([^;]+);U:\s+([^;]+);T:\s+([^\]]+)\]$/<IMG SRC="$1" ALIGN=LEFT HSPACE=20 ALT="$1"><a href="$2">$3<\/a><BR CLEAR=ALL>/i;
	# [u: 画像にリンクするURL;i: 画像URL;t: タイトル]
	$_[0] =~ s/^\[U:\s+([^;]+);I:\s+([^;]+);T:\s+([^\]]+)\]$/<a href="$1"><IMG SRC="$2" ALIGN=LEFT HSPACE=20 ALT="$2"><\/a>$3<BR CLEAR=ALL>/i;
	# [u: 画像にリンクするURL;i: 画像URL;u: タイトルにリンクするURL;t: タイトル]
	$_[0] =~ s/^\[U:\s+([^;]+);I:\s+([^;]+);U:\s+([^;]+);T:\s+([^\]]+)\]$/<a href="$1"><IMG SRC="$2" ALIGN=LEFT HSPACE=20 ALT="$2"><\/a><a href="$3">$4<\/a><BR CLEAR=ALL>/i;

	# 『』で囲まれた部分を引用部として処理する
	$_[0] =~ s/『([^』]+)』/『<cite>$1<\/cite>』/ig;

	# URL 文字列に URL 文字列の表すハイパーリンクを張る
	# URL 文字列の次の文字が URL 文字列を構成する文字列以外であり、
	# かつマークアップ記号ではない場合、
	$_[0] =~ s/(http:\/\/[#%a-z0-9\-_.!~*'\(\);\/?:@&=+\$,]+)([^<>"#%a-z0-9\-_.!~*'\(\);\/?:@&=+\$,]+)/<a href="$1">$1<\/a>$2/ig;
	# URL 文字列の並びの次が行末である場合、
	$_[0] =~ s/(http:\/\/[#%a-z0-9\-_.!~*'\(\);\/\?:@&=+\$,]+)$/<a href="$1">$1<\/a>/i;

	$catcount++;
	return $_[0];
}

# 現在の日付を yyyy-mm-dd 形式で得る
sub date{
	($mon = (localtime)[4] + 1) =~ s/^(\d)$/0$1/;
	($mday = (localtime)[3]) =~ s/^(\d)$/0$1/;
	$year = (localtime)[5] + 1900;
	return $year . "-" . $mon . "-" . $mday;
}

# <>実体参照エンコード（<pre>タグの中で使う）
sub entities_encode{
  my($str) = @_;
  $str =~ s/</&lt;/g;
  $str =~ s/>/&gt;/g;
  return $str;
}

# URLエンコーディング
sub jurl_encode{
  my($str) = @_;
  $str =~ s/([^a-z0-9\-_.!*'\(\)~ ])/sprintf("%%%02X", ord($1))/egi;
  $str =~ s/ /+/g;
  return $str;
}
