#!/usr/local/bin/perl

=head1 スクリプト名

wemo_edit.cgi - メモ作成編集（Wemo システムの CGI の一つ）

=head1 概要

HTML フォーム等からの変数入力に基づいて、wemo 言語による HTML メモのテキストフォームを生成する。保存時に、wemo_save.cgi を起動し、メモテキストを生成すると同時にHTMLに変換して表示する。

=cut
use CGI qw(:cgi);

# 環境変数の取得とメモを格納するディレクトリ名の設定
$docroot = "/homepage";#$ENV{'DOCROOT'};# HTTP サーバーのドキュメント格納ディレクトリ
$cgidir = "/cgi-bin";#$ENV{'CGIDIR'};# HTTP の CGI を格納する相対パス
$memodir = "memo";# メモ格納ディレクトリ名

# メモ入力 HTML 先頭部分 CGI 出力
print <<TOP;
Content-type: text/html; charset=UTF-8

<HTML>
<HEAD>
<TITLE>Wemo</TITLE>
<META HTTP-EQUIV="Content-Type" content="text/html; charset=UTF-8">
<link rel="stylesheet" type="text/css" href="/mystyle.css">
</HEAD>
<BODY>
<div class="emph">Wemo Edit</div>
TOP

# フォームのデータに基づいてメモファイル名を設定する
# param('today') の値が yes であれば今日の日付のメモを作成する
# param('date') の値があれば、その値の日付のメモを作成する
# いずれでもなければ、現在の日付のメモを指定する
if(param('today') eq 'yes'){
	$date = &date;
}elsif(param('selecteddate')){
	$date = param('selecteddate');
}elsif(param('date')){
	$date = param('date');
}elsif($ENV{'PATH_INFO'}){
	($date = $ENV{'PATH_INFO'}) =~ s/^\/memo_(\d{4}-\d{2}-\d{2})\.txt$/$1/i;
}
if($date !~ /^\d{4}-\d{2}-\d{2}$/){
	print "Wemo's date: ${date} is invalid!";
	exit;
}
$infile = "memo_${date}.txt";# メモファイル名の設定

# メモ入力フォーム先頭部分の CGI 出力
# メモファイル名 $infile は CGIのパス名の後に / で区切って渡される
# wemo.cgi は環境変数 PATH_INFO から /$infile として受け取る
print <<HEADER;
<form method="POST" action="$cgidir/wemo_save.cgi/$infile" target="main">
<textarea rows=15 cols=60 name="memo">
HEADER

# メモファイルがあれば読み込み、メモの日付行に続いて、wemo 言語を出力する
if(open(IN, "<$docroot/$memodir/$infile")){
	while(<IN>){
		print;
		if(/^(\d{4}-\d{2}-\d{2}) \(\w{3}\.\)$/i){
			$date = $1;
			&printwemo;
		}
	}
	close(IN);
# メモファイルがなければ、wemo 言語で日付行等を出力する
}else{
	print "da:\n";
	&printwemo;
}

# メモ入力フォーム最終部分を CGI 出力する
print <<FOOTER;
</textarea>
<br>
<input type="submit" value="Save">
<input type="reset" value="Reset">
</form>
</BODY>
</HTML>
FOOTER

sub printwemo{
	# memo.html の「今日」のラジオボタンからの入力があれば、
	if(param('today')){
		print "c: memo\n";
		print "t: \n\n";
	}elsif(param('rndate')){
		print "c: memo\n";
		print "t: ",param('rndate'),": ",param('title'),"\n";
		print "p: [u: ",param('rnurl'),";t: ",param('rndate'),": ",param('title'),"]\n";
	# memo.html の「指定する日」のテキストフォームからの入力があれば、
	}elsif(param('selecteddate')){
		print "c: memo\n";
		print "t: \n\n";
	# 「お気に入り」検索結果から起動する場合、
	# date、url、title のデータを受け取る
	}elsif(param('url')){
		print "c: favor\n";
		print "t: ",param('title'),"\n";
		print "p: [u: ",param('url'),"]\n\n";
	# 「マイドキュメント」検索結果から起動する場合、
	# date、docurl、title のデータを受け取る
	}elsif(param('docurl')){
		print "c: file\n";
		print "t: ",param('title'),"\n";
		print "p: [u: ",param('docurl'),"]\n\n";
	# 「註釈を書く」のリンクから起動する場合、
	# annot、title、name のデータを受け取る
	}elsif(param('annot')){
		print "c: annot\n";
		print "t: ",param('annot'),":「",param('title'),"」のメモへの註釈\n";
		print "p: [u: /$cgidir/wemo.cgi/memo_",param('annot'),".txt#",param('name'),"]\n\n";
	# メモカレンダーから起動する場合、
	# date、sched のデータを受け取る
	}elsif(param('sched')){
		print "c: sched\n";
		print "t: \n";
		print "p: \n\n";
	# メモ作成・編集 HTML のメニュー選択フォームから起動する場合、
	# category のデータを受け取る
	}elsif(param('category')){
		print "c: ",param('category'),"\n";
		print "t: \n";
		print "p: \n\n";
	}
}

# yyyy-mm-dd 形式で現在の日付を返す
sub date{
	my($mday,$mon,$year) = (localtime)[3..5];
	$mon += 1;
	$year += 1900;
	return sprintf("%4d-%1.2d-%1.2d", $year, $mon, $mday);
}
